'use strict';

exports.__esModule = true;
exports.addNode = exports.deleteNode = exports.udpateNode = exports.deleteNodeFromTree = exports.replaceNodeFromTree = exports.getNodeRenderOptions = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _reselect = require('reselect');

var _contants = require('../contants');

var getNodeRenderOptions = exports.getNodeRenderOptions = (0, _reselect.createSelector)(function (node) {
  return (node.state || {}).expanded;
}, function (node) {
  return (node.state || {}).favorite;
}, function (node) {
  return (node.state || {}).deletable;
}, function (node) {
  return node.children;
}, function (expanded, favorite, deletable) {
  var children = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
  return {
    hasChildren: !!children.length,
    isExpanded: !!expanded,
    isFavorite: !!favorite,
    isDeletable: !!deletable
  };
});

var replaceNodeFromTree = exports.replaceNodeFromTree = function replaceNodeFromTree(nodes, updatedNode) {
  return nodes.map(function (node) {
    if (node.id === updatedNode.id) {
      return updatedNode;
    }

    if (node.children) {
      return _extends({}, node, { children: replaceNodeFromTree(node.children, updatedNode) });
    }

    return node;
  });
};

var findNodeOnTree = function findNodeOnTree(parents, parentNode, id) {
  if (!parents.length) {
    return {
      node: parentNode.children.find(function (n) {
        return n.id === id;
      }),
      parent: parentNode
    };
  }

  var nextParent = parents[0];


  return findNodeOnTree(parents.slice(1), parentNode.children.find(function (n) {
    return n.id === nextParent;
  }), id);
};

var isNodeInTreeRoot = function isNodeInTreeRoot(n) {
  return n.id === undefined;
};

var deleteNodeFromTree = exports.deleteNodeFromTree = function deleteNodeFromTree(nodes, deletedNode) {
  var _findNodeOnTree = findNodeOnTree(deletedNode.parents, { children: nodes }, deletedNode.id),
      parent = _findNodeOnTree.parent;

  var childIndex = parent.children.findIndex(function (c) {
    return c.id === deletedNode.id;
  });

  if (isNodeInTreeRoot(parent)) {
    return [].concat(nodes.slice(0, childIndex), nodes.slice(childIndex + 1));
  }

  return replaceNodeFromTree(nodes, _extends({}, parent, {
    children: [].concat(parent.children.slice(0, childIndex), parent.children.slice(childIndex + 1))
  }));
};

var udpateNode = exports.udpateNode = function udpateNode(originalNode, newState) {
  return {
    node: _extends({}, originalNode, {
      state: _extends({}, originalNode.state, newState)
    }),
    type: _contants.UPDATE_TYPE.UPDATE
  };
};

var deleteNode = exports.deleteNode = function deleteNode(node) {
  return {
    node: node,
    type: _contants.UPDATE_TYPE.DELETE
  };
};

var addNode = exports.addNode = function addNode(node) {
  return {
    node: node,
    type: _contants.UPDATE_TYPE.ADD
  };
};